@@ -9,11 +9,9 @@ from functools import partial, wraps
 
 try:
     str = unicode  # Python 2.* compatible
-    PY3 = False
     string_types = (basestring,)
     integer_types = (int, long)
 except NameError:
-    PY3 = True
     string_types = (str,)
     integer_types = (int,)
 
@@ -73,7 +71,7 @@ class Comparable(object):
     __slots__ = ()
 
     def _c(op, inv=False):
-        return (lambda self, other: Condition(op, self, other)) if not inv else (lambda self, other: Condition(op, other, self))
+        return (lambda self, other: Condition(self, op, other)) if not inv else (lambda self, other: Condition(other, op, self))
 
     def _ca(op, inv=False):
         return (lambda self, *a: Constant(op)(self, *a)) if not inv else (lambda self, other: Constant(op)(other, self))
@@ -112,6 +110,8 @@ class Comparable(object):
     __lt__ = _c("<")
     __ge__ = _c(">=")
     __le__ = _c("<=")
+    is_ = _c("IS")
+    is_not = _c("IS NOT")
     like = _c("LIKE")
     ilike = _c("ILIKE")
     rlike = _c("LIKE", 1)
@@ -144,32 +144,28 @@ class Comparable(object):
 
     def __eq__(self, other):
         if other is None:
-            return Condition("IS", self, Constant("NULL"))
+            return self.is_(None)
         if hasattr(other, '__iter__'):
             return self.in_(other)
-        return Condition("=", self, other)
+        return Condition(self, "=", other)
 
     def __ne__(self, other):
         if other is None:
-            return Condition("IS NOT", self, Constant("NULL"))
+            return self.is_not(None)
         if hasattr(other, '__iter__'):
             return self.not_in(other)
-        return Condition("<>", self, other)
+        return Condition(self, "<>", other)
 
     def as_(self, alias):
         return Alias(alias, self)
 
     def in_(self, other):
         if not isinstance(other, Expr) and hasattr(other, '__iter__'):
-            if len(other) < 1:
-                raise Error("Empty list is not allowed")
             other = ExprList(*other).join(", ")
         return ExprList(self, Constant("IN"), Parentheses(other)).join(" ")
 
     def not_in(self, other):
         if not isinstance(other, Expr) and hasattr(other, '__iter__'):
-            if len(other) < 1:
-                raise Error("Empty list is not allowed")
             other = ExprList(*other).join(", ")
         return ExprList(self, Constant("NOT IN"), Parentheses(other)).join(" ")
 
@@ -182,11 +178,17 @@ class Comparable(object):
     def concat_ws(self, sep, *args):
         return Concat(self, *args).ws(sep)
 
+    def op(self, op):
+        return lambda other: Condition(self, op, other)
+
+    def rop(self, op):  # useless, can be P('lookingfor').op('=')(expr)
+        return lambda other: Condition(other, op, self)
+
     def asc(self):
-        return Suffix(self, "ASC")
+        return Postfix(self, "ASC")
 
     def desc(self):
-        return Suffix(self, "DESC")
+        return Postfix(self, "DESC")
 
     def __getitem__(self, key):
         """Returns self.between()"""
@@ -221,33 +223,27 @@ class Expr(Comparable):
 
 class Condition(Expr):
 
-    __slots__ = ('_op', '_expr1', '_expr2')
+    __slots__ = ('_left', '_right')
 
-    def __init__(self, op, expr1, expr2):
-        self._op = op.upper()
-        self._expr1 = None if expr1 is None else prepare_expr(expr1)
-        self._expr2 = None if expr2 is None else prepare_expr(expr2)
+    def __init__(self, left, op, right):
+        self._left = prepare_expr(left)
+        self._sql = op.upper()
+        self._right = prepare_expr(right)
 
     def __sqlrepr__(self, dialect):
-        s1 = sqlrepr(self._expr1, dialect)
-        s2 = sqlrepr(self._expr2, dialect)
-        if not s1:
-            return s2
-        if not s2:
-            return s1
-        return "{0} {1} {2}".format(s1, self._op, s2)
+        return "{0} {1} {2}".format(sqlrepr(self._left, dialect), self._sql, sqlrepr(self._right, dialect))
 
     def __params__(self):
-        params = sqlparams(self._expr1) + sqlparams(self._expr2)
+        params = sqlparams(self._left) + sqlparams(self._right)
         return params
 
 
 class ExprList(Expr):
 
-    __slots__ = ('_sep', '_args')
+    __slots__ = ('_args', )
 
     def __init__(self, *args):
-        self._sep = " "
+        self._sql = " "
         self._args = []
         args = list(args)
         if len(args) and hasattr(args[0], '__iter__'):
@@ -258,7 +254,7 @@ class ExprList(Expr):
             self._args[i] = prepare_expr(arg)
 
     def join(self, sep):
-        self._sep = sep
+        self._sql = sep
         return self
 
     def __len__(self):
@@ -300,11 +296,7 @@ class ExprList(Expr):
         return self
 
     def __sqlrepr__(self, dialect):
-        sqls = []
-        for a in self._args:
-            sql = sqlrepr(a, dialect)
-            sqls.append(sql)
-        return self._sep.join(sqls)
+        return self._sql.join([sqlrepr(a, dialect) for a in self._args])
 
     def __params__(self):
         params = []
@@ -323,7 +315,7 @@ class FieldList(ExprList):
     __slots__ = ()
 
     def _build(self):
-        sql = ExprList().join(self._sep)
+        sql = ExprList().join(self._sql)
         for a in self._args:
             if isinstance(a, Alias):
                 a = ExprList(a._expr, Constant("AS"), a).join(" ")
@@ -338,28 +330,28 @@ class FieldList(ExprList):
 
 class Concat(ExprList):
 
-    __slots__ = ('_sep', '_args', '_ws')
+    __slots__ = ('_args', '_ws')
 
     def __init__(self, *args):
         super(Concat, self).__init__(*args)
-        self._sep = ' || '
+        self._sql = ' || '
         self._ws = None
 
     def ws(self, sep):
         self._ws = prepare_expr(sep)
         return self
 
+    def __sqlrepr__(self, dialect):
+        value = sqlrepr(self, dialect, ExprList)
+        return "concat_ws({0}, {1})".format(sqlrepr(self._ws, dialect), value) if self._ws else value
+
     def __params__(self):
         return sqlparams(self._ws) + super(Concat, self).__params__()
 
-    def __sqlrepr__(self, dialect):
-        value = super(Concat, self).__sqlrepr__(dialect)
-        return "concat_ws({0}, {1})".format(self._ws, value) if self._ws else value
-
 
 class Placeholder(Expr):
 
-    __slots__ = ('_sql', '_params')
+    __slots__ = ()
 
     def __init__(self, *params):
         super(Placeholder, self).__init__(PLACEHOLDER, *params)
@@ -386,29 +378,29 @@ class OmitParentheses(Parentheses):
 
 class Prefix(Expr):
 
-    __slots__ = ('_prefix', '_expr', )
+    __slots__ = ('_expr', )
 
     def __init__(self, prefix, expr):
-        self._prefix = prefix
+        self._sql = prefix
         self._expr = prepare_expr(expr)
 
     def __sqlrepr__(self, dialect):
-        return "{0} {1}".format(self._prefix, sqlrepr(self._expr, dialect))
+        return "{0} {1}".format(self._sql, sqlrepr(self._expr, dialect))
 
     def __params__(self):
         return sqlparams(self._expr)
 
 
-class Suffix(Expr):
+class Postfix(Expr):
 
-    __slots__ = ('_suffix', '_expr', )
+    __slots__ = ('_expr', )
 
-    def __init__(self, expr, suffix):
-        self._suffix = suffix
+    def __init__(self, expr, postfix):
+        self._sql = postfix
         self._expr = prepare_expr(expr)
 
     def __sqlrepr__(self, dialect):
-        return "{0} {1}".format(sqlrepr(self._expr, dialect), self._suffix)
+        return "{0} {1}".format(sqlrepr(self._expr, dialect), self._sql)
 
     def __params__(self):
         return sqlparams(self._expr)
@@ -493,7 +485,7 @@ class Field(MetaField(bytes("NewBase"), (Expr, ), {})):
     def __sqlrepr__(self, dialect):
         sql = self._name == '*' and self._name or qn(self._name, dialect)
         if self._prefix is not None:
-            sql = ".".join((qn(self._prefix, dialect), sql, ))
+            sql = ".".join((qn(self._prefix, dialect), sql))
         return sql
 
 
@@ -563,7 +555,7 @@ class TableAlias(Table):
 
     __slots__ = ('_table', '_alias')
 
-    def __init__(self, alias, table):
+    def __init__(self, alias, table=None):
         self._table = table
         self._alias = alias
 
@@ -796,7 +788,7 @@ class QuerySet(Expr):
             if hasattr(args[0], '__iter__'):
                 self = self.order_by(*args.pop(0), reset=True)
             direct = "DESC" if opts.get("desc") else "ASC"
-            self._order_by.extend([f if isinstance(f, Suffix) and f._suffix in ("ASC", "DESC") else Suffix(f, direct) for f in args])
+            self._order_by.extend([f if isinstance(f, Postfix) and f._sql in ("ASC", "DESC") else Postfix(f, direct) for f in args])
         return self
 
     def limit(self, *args, **kwargs):
@@ -1027,10 +1019,10 @@ def default_dialect(dialect=None):
     return DEFAULT_DIALECT
 
 
-def sqlrepr(obj, dialect=None):
+def sqlrepr(obj, dialect=None, cls=None):
     """Renders query set"""
     dialect = dialect or DEFAULT_DIALECT
-    callback = sql_dialects.sqlrepr(dialect, obj.__class__)
+    callback = sql_dialects.sqlrepr(dialect, cls or obj.__class__)
     if callback is not None:
         return callback(obj, dialect)
     return obj  # It's a string
@@ -1046,16 +1038,11 @@ def sqlparams(obj):
 def warn(old, new, stacklevel=3):
     warnings.warn("{0} is deprecated. Use {1} instead".format(old, new), PendingDeprecationWarning, stacklevel=stacklevel)
 
-T, TA, F, A, E, QS = Table, TableAlias, Field, Alias, Expr, QuerySet
+A, C, E, F, P, T, TA, QS = Alias, Condition, Expr, Field, Placeholder, Table, TableAlias, QuerySet
 func = const = ConstantSpace()
 qn = Name()
 
 for cls in (Expr, Table, TableJoin, ):
-    cls.__bytes__ = lambda self: sqlrepr(self).encode('utf-8')
-    cls.__str__ = lambda self: sqlrepr(self)
     cls.__repr__ = lambda self: "<{0}: {1}, {2}>".format(type(self).__name__, sqlrepr(self), sqlparams(self))
-    if not PY3:
-        cls.__unicode__ = cls.__str__
-        cls.__str__ = cls.__bytes__
 
 from . import dialects
