@@ -1,5 +1,5 @@
 from __future__ import absolute_import, unicode_literals
-from . import (Condition, Name, Concat, sql_dialects, sqlrepr)
+from . import (Condition, Concat, ExprList, Name, sql_dialects, sqlrepr)
 
 TRANSLATION_MAP = {
     'sqlite': {
@@ -17,14 +17,8 @@ TRANSLATION_MAP = {
 @sql_dialects.register('mysql', Condition)
 def condition_sqlrepr(self, dialect):
     """Translates operators for sqlbuilder.smartsql.Condition"""
-    s1 = sqlrepr(self._expr1, dialect)
-    s2 = sqlrepr(self._expr2, dialect)
-    op = TRANSLATION_MAP.get(dialect, {}).get(self._op, self._op)
-    if not s1:
-        return s2
-    if not s2:
-        return s1
-    return "{0} {1} {2}".format(s1, op, s2)
+    op = TRANSLATION_MAP.get(dialect, {}).get(self._sql, self._sql)
+    return "{0} {1} {2}".format(sqlrepr(self._left, dialect), op, sqlrepr(self._right, dialect))
 
 
 @sql_dialects.register('sqlite', Name)
@@ -35,7 +29,7 @@ def name_sqlrepr(self, dialect):
 
 @sql_dialects.register('mysql', Concat)
 def name_sqlrepr(self, dialect):
-    value = super(Concat, self.join(', ')).__sqlrepr__(dialect)
+    value = sqlrepr(self.join(', '), dialect, ExprList)
     if self._ws:
-        return "CONCAT_WS({0}, {1})".format(self._ws, value)
+        return "CONCAT_WS({0}, {1})".format(sqlrepr(self._ws, dialect), value)
     return "CONCAT({0})".format(value)
